import React, {Component} from "react";
import StarRatingComponent from "react-star-rating-component";
import {Button, Icon, NavBar, Toast} from "antd-mobile";
import AttUtil from "../AttUtil/AttUtil";

class SignCanvas extends Component {

  state = {
    originStarData: 0,
    originSignData: '',
    starData: 0,
    signData: ''
  };

  componentDidMount() {
    const {signs, stars} = this.props;
    this.setState({
      originStarData: stars || 0,
      originSignData: signs || '',
      signData: signs,
      starData: stars
    });
  }

  //初始化画布
  initialCanvas = () => {
    const {signCanvas} = this.refs;
    const ctx = signCanvas.getContext("2d");
    signCanvas.width = signCanvas.width;
    // 如果signDatas有值，则显示
    const {signData} = this.state;
    if (signData) {
      const imgObj = new Image();
      imgObj.src = AttUtil.PIC_PREFIX + signData;
      //待图片加载完后，将其显示在canvas上
      imgObj.onload = () => {
        //this即是imgObj,保持图片的原始大小：470*480
        ctx.drawImage(imgObj, 0, 0);
      };
    }
    ctx.strokeStyle = 'Black';
    ctx.lineWidth = 1;
    const drawer = {
      isDrawing: false,
      touchstart: (coors) => {
        ctx.beginPath();
        ctx.moveTo(coors.x, coors.y);
        drawer.isDrawing = true;
      },
      touchmove: (coors) => {
        if (drawer.isDrawing) {
          ctx.lineTo(coors.x, coors.y);
          ctx.stroke();
        }
      },
      touchend: (coors) => {
        if (drawer.isDrawing) {
          drawer.touchmove(coors);
          drawer.isDrawing = false;
        }
      }
    };
    const draw = (event) => {
      if (!event.targetTouches[0]) {
        ctx.closePath();
        drawer.isDrawing = false;
        return;
      }
      const coors = {
        x: event.targetTouches[0].pageX,
        y: event.targetTouches[0].pageY
      };
      let obj = signCanvas;
      if (obj.offsetParent) {
        do {
          coors.x -= obj.offsetLeft;
          coors.y -= obj.offsetTop;
        } while ((obj = obj.offsetParent) != null);
      }
      drawer[event.type](coors);
    };
    signCanvas.addEventListener('touchstart', draw, false);
    signCanvas.addEventListener('touchmove', draw, false);
    signCanvas.addEventListener('touchend', draw, false);
    signCanvas.addEventListener('touchmove', (event) => {
      event.preventDefault();
    }, false);
  };

  //显示画布
  showCanvas = () => {
    const {signature} = this.refs;
    this.initialCanvas();
    signature.style.animation = 'signature_appear 0.4s linear';
    signature.style.display = 'block';
  };

  //隐藏画布
  hideCanvas = () => {
    const {signature} = this.refs;
    signature.style.display = 'none';
  };

  //保存签字内容
  save = () => {
    const {signCanvas} = this.refs;
    const {onChange, useStar} = this.props;
    const {starData} = this.state;
    if(useStar && !starData){
      Toast.info(`请进行客户评价`, AttUtil.TOAST_DURATION);
      return
    }
    const signData = signCanvas.toDataURL("image/png");
    const index = signData.lastIndexOf("base64");
    const signBase64 = signData.substring(index + 7, signData.length);
    this.setState({
      originStarData: starData,
      originSignData: signBase64,
      signData: signBase64
    });
    this.hideCanvas();
    onChange ? onChange(false, signBase64, starData) : '';
  };

  //清楚签字内容
  clear = () => {
    const {signData,starData} = this.state;
    this.setState({
      originStarData: starData,
      originSignData: signData,
      starData: 0,
      signData: ''
    });
    const {signCanvas} = this.refs;
    signCanvas.width = signCanvas.width;
  };

  //返回
  back = () => {
    const {originStarData, originSignData} = this.state;
    const {onChange} = this.props;
    const {signature} = this.refs;
    signature.style.display = 'none';
    this.setState({
      starData: originStarData,
      signData: originSignData
    });
    onChange(false);
    this.hideCanvas();
  };

  //星级点击事件
  onStarClick(nextValue) {
    this.setState({
      starData: nextValue
    })
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    const {isSigning} = this.props;
    const wasSigning = prevProps.isSigning;
    if(!wasSigning&&isSigning){
      isSigning ? this.showCanvas() : this.hideCanvas();
    }
  }

  render() {
    const {starData} = this.state;
    const {useStar} = this.props;
    return <div ref='signature' className={`zk_attachments_signature`}>
      <NavBar
        mode='dark'
        icon={<Icon type='left' size='lg'/>}
        leftContent={'返回'}
        onLeftClick={this.back}
      >签字</NavBar>
      {useStar ? <div className={'zk_attachments_star'}>
        <div>客户评价</div>
        <StarRatingComponent
          name="attachments-starRating"
          starCount={5}
          value={starData}
          onStarClick={(nextValue) => {
            this.onStarClick(nextValue)
          }}
        />
      </div> : ''}
      <canvas width={window.screen.width} height={document.body.scrollHeight - 95 - (useStar ? 45 : 0)}
              ref='signCanvas'/>
      <Button className={'zk_attachments_foot'} onClick={() => {
        this.clear();
      }}>清 除</Button>
      <Button className={'zk_attachments_foot'} type={'primary'} onClick={() => {
        this.save();
      }}>保 存</Button>
    </div>
  }
}

export default SignCanvas;
